#!/bin/bash
#
# ACME PKI (Certificate) Bootstrap v1.3
# 
# This script will generate all the files necessary to build a certificate chain of trust
# using a CA, CA Intermediate, Server, and Client certificates. After the bootstrap the other
# helper scripts will generate new client/server certificates
# 

# source this file to include the functions
. libs/pki_funcs.sh

PARAM1=$1

usage() {
  echo
  echo "This application will generate all the files necessary to build a certificate chain of trust"
  echo "using a CA, CA Intermediate, Server, and Client certificates. All the files are put into"
  echo "pki lifecyle package"
  echo "  -put the .cnf config files into the ./cnf directory"
  echo 
  echo "Usage: pki_bootstrap  <.cnf file (minus the .cnf)>"
  echo
  echo "Example: pki_bootstrap  org.acme.xyz"
  exit 1
}

#
# CA generation requires .cnf files
# create CA directory
# create bash variables to CA
# restore script back to original path
#
app_init() {
  if [[ -n $PARAM1 ]]; then
    # need to know the location of the configuration file  (expected to be in same dir path as this script)
    CA_CNF="$CD_ROOT/cnf/ca.cnf"

    # handle the case of having the ".cnf" extension or not
    if [[ ${PARAM1: -4} == .cnf ]]; then
      ORG_URL=${PARAM1%.*}
      S_CNF=${PARAM1}
      echo "ASDF: ${ORG_URL},  ${S_CNF}"
    else
      ORG_URL=$PARAM1
      S_CNF="${PARAM1}.cnf"
      echo "ZXCV: ${ORG_URL},  ${S_CNF}"
    fi

    FQ_S_CNF="${CD_ROOT}/cnf/${S_CNF}"
    if [[ ! -f $FQ_S_CNF ]] || [[ ! -f $CA_CNF ]]; then
      usage
    fi
  else
    usage
  fi
}

#
# IN: UNIQ_ID_CA, SERIAL
#
one-time-ca() {
  # params
  #SERIAL="101"

  get_serial
  echo_block "SERIAL == ${SERIAL}"
  # Organize
  # 
  # create a unique path for the server certificate
  UNIQ_DIR_LC=`date +%Y-%m-%d.%H_%M_%S`
  UNIQ_DIR_LC="pki-lifecycle_${UNIQ_DIR_LC}"
  mkdir -p "${UNIQ_DIR_LC}"
  cd "${UNIQ_DIR_LC}"

  # create certificate
  UNIQ_ID_CA="${SERIAL}.${ORG_URL}"
  CA_DIR="ca_${UNIQ_ID_CA}"
  mkdir $CA_DIR
  cd $CA_DIR
  FQ_CA_DIR=`pwd`
  FQ_CA_CERT="${FQ_CA_DIR}/ca_${UNIQ_ID_CA}.crt.pem"
  FQ_CA_KEYS="${FQ_CA_DIR}/ca_${UNIQ_ID_CA}.keys.pem"
  generate_ca $UNIQ_ID_CA $SERIAL
  cd ..
}

#
# Organize the files into logical folders based on serial #
#
organize() {
  # organize the client directory
  mkdir -p clients/ca-i
  mkdir -p clients/data
  mkdir -p clients/distro
  mkdir -p clients/docs
  mv client*.pem clients/data/
  mv client*.p12 clients/distro/
  mv client*.info.txt clients/docs/
  cp ca_i*.crt.pem clients/ca-i/
  cp ca_i*.keys.pem clients/ca-i/

  # organize the server directory
  mkdir -p servers/ca-i
  mkdir -p servers/data
  mkdir -p servers/distro
  mkdir -p servers/docs
  mv server_*.pem servers/data/
  mv server_*.p12 servers/distro/
  mv server_*.info.txt servers/docs/
  cp ca_i*.crt.pem servers/ca-i/
  cp ca_i*.keys.pem servers/ca-i/

  # organize the ca-i directory
  mkdir -p ca-i/data
  mkdir -p ca-i/docs
  mv ca_i*.pem ca-i/data/
  mv ca_i*.info.txt ca-i/docs/
  mv ca_i*.p12 ca-i/
  mv ca_cert-chain*.pem ca-i/
  cp $FQ_CA_DIR/ca_*.crt.pem ca-i/data/
  cp $FQ_CA_DIR/ca_*.info.txt ca-i/docs/
}

#
# Copies all applcations to the Lifecycle package
# 
# Requires: 
#   UNIQ_DIR_LC  : unique string for the Lifecycle directory
#   UNIQ_ID_CA-I : unique string for the CA-I
#
cp_pki_lifecycle() {
  cp $CD_ROOT/libs/gen_ca-i.sh $CD_ROOT/$UNIQ_DIR_LC/
  cp $CD_ROOT/docs/README_LC $CD_ROOT/$UNIQ_DIR_LC/README
  cp $CD_ROOT/libs/gen_client.sh $CD_ROOT/$UNIQ_DIR_LC/distrobution/$UNIQ_DIR_CA/clients
  cp $CD_ROOT/libs/gen_server.sh $CD_ROOT/$UNIQ_DIR_LC/distrobution/$UNIQ_DIR_CA/servers

  # cp $LIB_PATH/gen_ca-i.sh  $UNIQ_ID_CA-I/$UNIQ_DIR_LC  ca-i/
  # cp $LIB_PATH/gen_client.sh clients/
  # cp $LIB_PATH/gen_server.sh servers/
}

#
# Generate a PKI chain
#   - the certificate chain is unique based on the serial #
#   - generate a new CA I
#   - generate two server certificates
#   - generate two client certificates
#
# INPUT:  BASE SERIAL #, LOOP NUM
# 
gen_pki_certs() {
  B_SERIAL=$1
  NUM_CERTS=$(($2-1))

  # Create CA Intermediate
  UNIQ_ID_CAI="${B_SERIAL}.${ORG_URL}" 
  generate_ca_i $UNIQ_ID_CAI $B_SERIAL

  # Server Certificates
  for NUM in $(seq 0 $NUM_CERTS)
  do
    generate_server "$((B_SERIAL+NUM)).${ORG_URL}" $UNIQ_ID_CAI $((B_SERIAL+NUM))
  done

  # Client Certificates
  for NUM in $(seq 0 $NUM_CERTS)
  do
    generate_client "$((B_SERIAL+NUM)).${ORG_URL}" $UNIQ_ID_CAI $((B_SERIAL+NUM))
  done
}

#
# INPUT:  SERIAL #, LOOP NUM
# 
gen_pki() {
  # organization
  CDD=`pwd`

  SERIAL=$1
  UNIQ_DIR_CA="ca_i_${SERIAL}.${ORG_URL}"
  mkdir -p "distrobution/${UNIQ_DIR_CA}"
  cd "distrobution/${UNIQ_DIR_CA}"

  # geneate certificates, organize the files
  gen_pki_certs $SERIAL $2
  organize
  cp_pki_lifecycle

  # return to last path
  cd $CDD
}


main() {
  CD_ROOT=`pwd`
  LIB_PATH="${CD_ROOT}/libs"

  app_init
  one-time-ca
  gen_pki 1001 2
#  gen_pki 50001 5
#  gen_pki 80001 10
  
  # make sure we return to root execution path
  cd "${CD_ROOT}"
}


# ***** ***** ***** ***** ***** ***** ***** ***** ***** ***** ***** *****
# 
# main execution begins here  (because all the functions have to be defined)
# 
# ***** ***** ***** ***** ***** ***** ***** ***** ***** ***** ***** *****

main

# ***** ***** ***** *****
# 
#
# 
# ***** ***** ***** *****
